@@ -6,7 +6,7 @@ import logging
 
 import numpy as np
 
-from osl_dynamics import array_ops
+from osl_dynamics import array_ops, inference
 
 _logger = logging.getLogger("osl-dynamics")
 
@@ -164,58 +164,6 @@ def reverse_pca(covariances, pca_components):
     return pca_components @ covariances @ pca_components.T
 
 
-def partial_covariances(data, alpha):
-    """Calculate partial covariances.
-
-    Returns the multiple regression parameters estimates, pcovs, of the state
-    time courses regressed onto the data from each channel. The regression is
-    done separately for each channel. I.e. pcovs is the estimate of the
-    (n_modes, n_channels) matrix, Beta. We fit the regression Y_i = X @ Beta_i + e,
-    where:
-
-    - Y_i is (n_samples, 1) the data amplitude/envelope/power/abs time course at channel i.
-    - X is (n_samples, n_modes) matrix of the variance normalised mode time courses (i.e. alpha).
-    - Beta_i is (n_modes, 1) vector of multiple regression parameters for channel i.
-    - e is the error.
-
-    NOTE: state time courses are variance normalised so that all amplitude info goes
-    into Beta (i.e. pcovs).
-
-    Parameters
-    ----------
-    data : np.ndarray or list of np.ndarray
-        Training data for each subject.
-    alpha : np.ndarray or list of np.ndarray
-        State/mode time courses for each subject.
-
-    Returns
-    -------
-    pcovs : np.ndarray
-        Matrix of partial covariance (multiple regression parameter estimates).
-        Shape is (n_modes, n_channels).
-    """
-    if type(data) != type(alpha):
-        raise ValueError(
-            "data and alpha must be the same type: numpy arrays or lists of numpy arrays."
-        )
-    if isinstance(data, np.ndarray):
-        data = [data]
-        alpha = [alpha]
-    for i in range(len(data)):
-        if data[i].shape[0] != alpha[i].shape[0]:
-            raise ValueError("Difference number of samples in data and alpha.")
-
-    pcovs = []
-    for X, a in zip(data, alpha):
-        # Variance normalise state/mode time courses
-        a /= np.std(a, axis=0, keepdims=True)
-
-        # Do multiple regression of alpha onto data
-        pcovs.append(np.linalg.pinv(a) @ X)
-
-    return np.squeeze(pcovs)
-
-
 def state_activation(state_time_course):
     """Calculate state activations from a state time course.
 
@@ -645,3 +593,108 @@ def simple_moving_average(data, window_length, step_size):
         mov_avg[n] = np.mean(mov_window, axis=0)
 
     return mov_avg
+
+
+def partial_covariances(data, alpha):
+    """Calculate partial covariances.
+
+    Returns the multiple regression parameters estimates, pcovs, of the state
+    time courses regressed onto the data from each channel. The regression is
+    done separately for each channel. I.e. pcovs is the estimate of the
+    (n_modes, n_channels) matrix, Beta. We fit the regression::
+
+        Y_i = X @ Beta_i + e
+
+    where:
+
+    - Y_i is (n_samples, 1) the data amplitude/envelope/power/abs time course at channel i.
+    - X is (n_samples, n_modes) matrix of the variance normalised mode time courses
+      (i.e. alpha).
+    - Beta_i is (n_modes, 1) vector of multiple regression parameters for channel i.
+    - e is the error.
+
+    NOTE: state time courses are variance normalised so that all amplitude info goes
+    into Beta (i.e. pcovs).
+
+    Parameters
+    ----------
+    data : np.ndarray or list of np.ndarray
+        Training data for each subject.
+    alpha : np.ndarray or list of np.ndarray
+        State/mode time courses for each subject.
+
+    Returns
+    -------
+    pcovs : np.ndarray
+        Matrix of partial covariance (multiple regression parameter estimates).
+        Shape is (n_modes, n_channels).
+    """
+    if type(data) != type(alpha):
+        raise ValueError(
+            "data and alpha must be the same type: numpy arrays or lists of numpy arrays."
+        )
+    if isinstance(data, np.ndarray):
+        data = [data]
+        alpha = [alpha]
+    for i in range(len(data)):
+        if data[i].shape[0] != alpha[i].shape[0]:
+            raise ValueError("Difference number of samples in data and alpha.")
+
+    pcovs = []
+    for X, a in zip(data, alpha):
+        # Variance normalise state/mode time courses
+        a /= np.std(a, axis=0, keepdims=True)
+
+        # Do multiple regression of alpha onto data
+        pcovs.append(np.linalg.pinv(a) @ X)
+
+    return np.squeeze(pcovs)
+
+
+def ae_hmm_networks(data, alpha):
+    """Calculate subject-specific AE-HMM networks.
+
+    Parameters
+    ----------
+    data : list of np.ndarray
+        Amplitude envelope data (i.e. after preparation).
+        Shape is (n_subjects, n_channels).
+    alpha : list of np.ndarray
+        Inferred state probabilities. Shape is (n_subjects, n_states).
+
+    Returns
+    -------
+    means : np.ndarray
+        Subject-specific mean activity maps. Shape is (n_subjects, n_states, n_channels).
+    aecs : np.ndarray
+        Subject-specific amplitude envelope correlation maps.
+        Shape is (n_subjects, n_states, n_channels, n_channels).
+    """
+    if type(data) != type(alpha):
+        raise ValueError(
+            "data and alpha must be the same type: numpy arrays or lists of numpy arrays."
+        )
+    if isinstance(data, np.ndarray):
+        data = [data]
+        alpha = [alpha]
+
+    # Hard classify the state probabilties
+    stc = inference.modes.argmax_time_courses(alpha)
+
+    # Number of subjects, channels and states
+    n_subjects = len(data)
+    n_channels = data[0].shape[1]
+    n_states = stc[0].shape[1]
+
+    # Calculate subject and state specific networks
+    _logger.info(f"Calculating AE networks")
+    means = np.empty([n_subjects, n_states, n_channels], dtype=np.float32)
+    aecs = np.empty([n_subjects, n_states, n_channels, n_channels], dtype=np.float32)
+    for i in range(n_subjects):
+        d = data[i][: stc[i].shape[0]]
+        for j in range(n_states):
+            x = d[stc[i][:, j] == 1]
+            means[i, j] = np.mean(x, axis=0)
+            aecs[i, j] = np.corrcoef(x, rowvar=False)
+
+    return means, aecs
