@@ -13,7 +13,7 @@ from shutil import rmtree
 import numpy as np
 from pqdm.threads import pqdm
 from scipy import signal
-from tqdm import tqdm
+from tqdm.auto import tqdm
 
 from osl_dynamics.data import processing, rw, tf
 from osl_dynamics.utils import misc
@@ -60,14 +60,28 @@ class Data:
         reject_by_annotation="omit" to remove segments marked as bad.
     sampling_frequency : float
         Sampling frequency of the data in Hz. This argument is optional.
+    mask_file : str
+        Path to mask file used to source reconstruct the data. This argument is
+        optional.
+    parcellation_file : str
+        Path to parcellation file used to source reconstruct the data. This argument
+        is optional.
     store_dir : str
         We don't read all the data into memory. Instead we create store them on
         disk and create memmaps (unless load_memmaps=False is passed).
         This is the directory to save memmaps to. Default is ./tmp.
         This argument is optional.
     n_embeddings : int
-        Number of embeddings. Can be passed if data has already been prepared.
-        This argument is optional.
+        Number of time-delay embeddings that have already been appleid to the data.
+        This argument is optional. It is useful to pass this argument if the data has
+        already been prepared.
+    n_window : int
+        Length of sliding window that has already been applied to the data. This
+        argument is optional. It is useful to pass this argument if the data has
+        already been prepared.
+    amplitude_envelope : bool
+        Is the data we're loading amplitude envelope data? This argument is optional.
+        It is useful to pass this argument if the data has already been prepared.
     time_axis_first : bool
         Is the input data of shape (n_samples, n_channels)? Default is True.
         If your data is in format (n_channels, n_samples), use
@@ -88,8 +102,12 @@ class Data:
         picks=None,
         reject_by_annotation=None,
         sampling_frequency=None,
+        mask_file=None,
+        parcellation_file=None,
         store_dir="tmp",
         n_embeddings=None,
+        n_window=None,
+        amplitude_envelope=None,
         time_axis_first=True,
         load_memmaps=True,
         n_jobs=1,
@@ -99,7 +117,11 @@ class Data:
         self.picks = picks
         self.reject_by_annotation = reject_by_annotation
         self.sampling_frequency = sampling_frequency
+        self.mask_file = mask_file
+        self.parcellation_file = parcellation_file
         self.n_embeddings = n_embeddings
+        self.n_window = n_window
+        self.amplitude_envelope = amplitude_envelope
         self.time_axis_first = time_axis_first
         self.load_memmaps = load_memmaps
         self.n_jobs = n_jobs
@@ -462,6 +484,9 @@ class Data:
                 + "Data(..., sampling_frequency=...) when creating the Data object."
             )
 
+        if n_window % 2 == 0:
+            raise ValueError("n_window must be an odd number.")
+
         # Save settings
         self.amplitude_envelope = True
         self.low_freq = low_freq
@@ -693,6 +718,7 @@ class Data:
         self,
         sequence_length=None,
         n_embeddings=1,
+        n_window=1,
         prepared=True,
         concatenate=False,
     ):
@@ -707,7 +733,9 @@ class Data:
         sequence_length : int
             Length of the segement of data to feed into the model.
         n_embeddings : int
-            Number of data points to embed the data.
+            Number of data points used to embed the data.
+        n_window : int
+            Number of data points the sliding window applied to the data.
         prepared : bool
             Should we return the prepared data? If not we return the raw data.
         concatenate : bool
@@ -718,14 +746,19 @@ class Data:
         list of np.ndarray
             Trimed time series for each subject.
         """
-        if self.n_embeddings is None:
+        if self.n_embeddings is None and self.n_window is None:
             # Data has not been prepared so we can't trim the prepared data
             prepared = False
 
         if not prepared:
-            # We're trimming the raw data, how many time embedding data
-            # points do we need to remove?
-            n_embeddings = self.n_embeddings or n_embeddings
+            # We're trimming the raw data, how many data points do we
+            # need to remove due to time embedding or moving average?
+            if self.amplitude_envelope:
+                n_remove = self.n_window or n_window
+            else:
+                n_remove = self.n_embeddings or n_embeddings
+        else:
+            n_remove = 1
 
         # What data should we trim?
         if prepared:
@@ -736,8 +769,8 @@ class Data:
         trimmed_time_series = []
         for memmap in memmaps:
             # Remove data points lost to time embedding
-            if n_embeddings != 1:
-                memmap = memmap[n_embeddings // 2 : -(n_embeddings // 2)]
+            if n_remove != 1:
+                memmap = memmap[n_remove // 2 : -(n_remove // 2)]
 
             # Remove data points lost to separating into sequences
             if sequence_length is not None:
