@@ -197,6 +197,36 @@ class InverseCholeskyLayer(layers.Layer):
         return self.bijector.inverse(inputs)
 
 
+class SampleGammaDistributionLayer(layers.Layer):
+    """Layer for sampling from a gamma distribution.
+
+    This layer accepts the shape and rate
+    and outputs samples from a gamma distribution.
+
+    Parameters
+    ----------
+    epsilon : float
+        Error to add to the shape and rate for numerical stability.
+    """
+
+    def __init__(self, epsilon, **kwargs):
+        super().__init__(**kwargs)
+        self.epsilon = epsilon
+
+    def call(self, inputs, training=None, **kwargs):
+        alpha, beta = inputs
+        alpha = add_epsilon(alpha, self.epsilon)
+        beta = add_epsilon(beta, self.epsilon)
+        if training:
+            N = tfp.distributions.Gamma(
+                concentration=alpha, rate=beta, allow_nan_stats=False
+            )
+            return N.sample()
+        else:
+            mode = (alpha - 1) / beta
+            return tf.maximum(mode, 0)
+
+
 class SampleNormalDistributionLayer(layers.Layer):
     """Layer for sampling from a normal distribution.
 
@@ -238,7 +268,7 @@ class SampleGumbelSoftmaxDistributionLayer(layers.Layer):
 
     def call(self, inputs, **kwargs):
         gs = tfp.distributions.RelaxedOneHotCategorical(
-            temperature=temperature, logits=inputs
+            temperature=self.temperature, logits=inputs
         )
         return gs.sample()
 
@@ -269,71 +299,117 @@ class SoftmaxLayer(layers.Layer):
         **kwargs,
     ):
         super().__init__(**kwargs)
-        self.initial_temperature = initial_temperature
-        self.learn_temperature = learn_temperature
-        self.temperature_initializer = osld_initializers.WeightInitializer(
-            self.initial_temperature
-        )
-
-    def build(self, input_shape):
-        self.temperature = self.add_weight(
-            "temperature",
-            shape=(),
-            dtype=tf.float32,
-            initializer=self.temperature_initializer,
-            trainable=self.learn_temperature,
-        )
-        self.built = True
+        self.layers = [
+            LearnableTensorLayer(
+                shape=(),
+                learn=learn_temperature,
+                initial_value=initial_temperature,
+                name=self.name + "_kernel",
+            )
+        ]
 
     def call(self, inputs, **kwargs):
-        return activations.softmax(inputs / self.temperature, axis=2)
+        temperature = self.layers[0](inputs)
+        return activations.softmax(inputs / temperature, axis=2)
 
 
-class ScalarLayer(layers.Layer):
-    """Layer to learn a single scalar.
+class LearnableTensorLayer(layers.Layer):
+    """Layer to learn a tensor.
 
     Parameters
     ----------
+    shape : tuple
+        Shape of the tensor.
     learn : bool
-        Should we learn the scalar?
+        Should we learn the tensor?
+    initializer : tf.keras.initializers.Initializer
+        Initializer for the tensor.
     initial_value : float
-        Initial value for the scalar.
+        Initial value for the tensor if initializer is not passed.
+    regularizer : osl-dynamics regularizer
+        Regularizer for the tensor. Must be from osl_dynamics.inference.regularizers.
     """
 
     def __init__(
         self,
-        learn,
-        initial_value,
+        shape,
+        learn=True,
+        initializer=None,
+        initial_value=None,
+        regularizer=None,
         **kwargs,
     ):
         super().__init__(**kwargs)
+
+        # Bool for if we're learning the tensor
         self.learn = learn
-        if initial_value is None:
-            self.initial_value = 1
+
+        # Shape of the tensor
+        if shape is None:
+            raise ValueError("A shape must be passed to LearnableTensorLayer.")
+        self.shape = shape
+
+        # Initial value of the tensor
+        self.initial_value = initial_value
+        if self.initial_value is not None:
+            self.initial_value = np.array(initial_value).astype(np.float32)
+
+        # Setup the tensor initializer
+        if initializer is None:
+            # Initializer not passed, use the initial value
+            if self.initial_value is None:
+                raise ValueError("initializer or initial_value must be passed.")
+            elif self.initial_value.shape != shape:
+                raise ValueError(
+                    "Shape of initial_value must match that of the tensor. "
+                    + f"Expected {shape}, got {self.initial_value.shape}."
+                )
+            self.tensor_initializer = osld_initializers.WeightInitializer(
+                self.initial_value
+            )
         else:
-            self.initial_value = initial_value
-        self.scalar_initializer = osld_initializers.WeightInitializer(
-            self.initial_value
-        )
+            # Use the initializer passed, initial_value is ignored
+            self.tensor_initializer = initializer
+
+        # Regularizer for the tensor
+        # This should be a function of the tensor that returns a float
+        self.regularizer = regularizer
+
+    def add_regularization(self, tensor, inputs):
+        # Calculate the regularisation from the tensor
+        reg = self.regularizer(tensor)
+
+        # Calculate the scaling factor for the regularization
+        # Note, inputs.shape[0] must be the batch size
+        batch_size = tf.cast(tf.shape(inputs)[0], tf.float32)
+        n_batches = self.regularizer.n_batches
+        scaling_factor = batch_size * n_batches
+        reg /= scaling_factor
+
+        # Add regularization to the loss and display while training
+        self.add_loss(reg)
+        self.add_metric(reg, name=self.name)
 
     def build(self, input_shape):
-        self.scalar = self.add_weight(
-            "scalar",
+        # Create a weight for the tensor
+        self.tensor = self.add_weight(
+            "tensor",
+            shape=self.shape,
             dtype=tf.float32,
-            initializer=self.scalar_initializer,
+            initializer=self.tensor_initializer,
             trainable=self.learn,
         )
         self.built = True
 
-    def call(self, inputs, **kwargs):
-        return self.scalar
+    def call(self, inputs, training=None, **kwargs):
+        if self.regularizer is not None and training:
+            self.add_regularization(self.tensor, inputs)
+        return self.tensor
 
 
 class VectorsLayer(layers.Layer):
     """Layer to learn a set of vectors.
 
-    The vectors are free parameters.
-
     Parameters
     ----------
     n : int
@@ -358,69 +434,39 @@ class VectorsLayer(layers.Layer):
         **kwargs,
     ):
         super().__init__(**kwargs)
-        self.n = n
-        self.m = m
-        self.learn = learn
 
         if initial_value is not None:
-            # Use value passed to initialise
-            if initial_value.ndim != 2:
-                raise ValueError(
-                    "a (n_modes, n_channels) array must be passed for initial_value."
-                )
-            if initial_value.shape[-1] != m:
-                raise ValueError(
-                    f"mismatch between the number of channels ({m}) and number of "
-                    + f"elements in initial_means ({initial_value.shape[-1]})."
-                )
-            if initial_value.shape[0] != n:
-                raise ValueError(
-                    "mismatch bettwen the number of modes and vectors in "
-                    + f"initial_means ({initial_value.shape[0]})."
-                )
+            # Check initial_value is the correct shape
+            if initial_value.shape != (n, m):
+                raise ValueError(f"initial_value shape must be ({n}, {m}).")
             initial_value = initial_value.astype("float32")
-            self.vectors_initializer = osld_initializers.WeightInitializer(
-                initial_value
-            )
 
-        elif learn:
-            # Use a random vector to initialise
-            self.vectors_initializer = initializers.TruncatedNormal(mean=0, stddev=0.02)
+            # We don't need an initializer
+            initializer = None
         else:
-            # We're not learning the vectors, use zeros
-            self.vectors_initializer = initializers.Zeros()
+            # No initial value has been passed, set the initializer
+            if learn:
+                initializer = initializers.TruncatedNormal(mean=0, stddev=0.02)
+            else:
+                initializer = initializers.Zeros()
 
-        # Regulariser
-        self.regularizer = regularizer
+        # We use self.layers for compatibility with
+        # initializers.reinitialize_model_weights
+        self.layers = [
+            LearnableTensorLayer(
+                shape=(n, m),
+                learn=learn,
+                initializer=initializer,
+                initial_value=initial_value,
+                regularizer=regularizer,
+                name=self.name + "_kernel",
+            )
+        ]
 
-    def add_regularization(self, inputs):
-        # Calculate regularisation
-        reg = self.regularizer(self.vectors)
-
-        # Calculate the scaling factor for the regularisation
-        batch_size = tf.cast(tf.shape(inputs)[0], tf.float32)
-        n_batches = self.regularizer.n_batches
-        scaling_factor = batch_size * n_batches
-        reg /= scaling_factor
-
-        # Add to the loss function
-        self.add_loss(reg)
-        self.add_metric(reg, name=self.name)
-
-    def build(self, input_shape):
-        self.vectors = self.add_weight(
-            "vectors",
-            shape=(self.n, self.m),
-            dtype=tf.float32,
-            initializer=self.vectors_initializer,
-            trainable=self.learn,
-        )
-        self.built = True
-
-    def call(self, inputs, training=None, **kwargs):
-        if self.regularizer is not None and training:
-            self.add_regularization(inputs)
-        return self.vectors
+    def call(self, inputs, **kwargs):
+        learnable_tensors_layer = self.layers[0]
+        vectors = learnable_tensors_layer(inputs, **kwargs)
+        return vectors
 
 
 class CovarianceMatricesLayer(layers.Layer):
@@ -457,82 +503,55 @@ class CovarianceMatricesLayer(layers.Layer):
         **kwargs,
     ):
         super().__init__(**kwargs)
-        self.n = n
-        self.m = m
-        self.learn = learn
         self.epsilon = epsilon
 
         # Bijector used to transform learnable vectors to covariance matrices
         self.bijector = tfb.Chain([tfb.CholeskyOuterProduct(), tfb.FillScaleTriL()])
 
+        # Do we have an initial value?
         if initial_value is not None:
-            # Used passed value to initialise
-            if initial_value.ndim != 3:
-                raise ValueError(
-                    "a (n_modes, n_channels, n_channels) array must be passed for "
-                    + "initial_covariances."
-                )
-            if initial_value.shape[-1] != m:
-                raise ValueError(
-                    f"mismatch between the number of channels ({m}) and number of "
-                    + "rows/columns in initial_covariances "
-                    + f"({initial_value.shape[-1]})."
-                )
-            if initial_value.shape[0] != n:
-                raise ValueError(
-                    "mismatch bettwen the number of modes and matrices in "
-                    + f"initial_covariances ({initial_value.shape[0]})."
-                )
+            # Check it's the correct shape
+            if initial_value.shape != (n, m, m):
+                raise ValueError(f"initial_value shape must be ({n}, {m}, {m}).")
+
+            # Calculate the flattened cholesky factors
             initial_value = initial_value.astype("float32")
             initial_flattened_cholesky_factors = self.bijector.inverse(initial_value)
-            self.flattened_cholesky_factors_initializer = (
-                osld_initializers.WeightInitializer(initial_flattened_cholesky_factors)
-            )
-
-        elif learn:
-            # Use the identity matrix with a random error
-            self.flattened_cholesky_factors_initializer = (
-                osld_initializers.NormalIdentityCholeskyInitializer(std=0.1)
-            )
 
+            # We don't need an initializer
+            initializer = None
         else:
-            # Use the identity matrix
-            self.flattened_cholesky_factors_initializer = (
-                osld_initializers.IdentityCholeskyInitializer()
+            # No initial value has been passed
+            initial_flattened_cholesky_factors = None
+            if learn:
+                # Use a random initializer
+                initializer = osld_initializers.NormalIdentityCholeskyInitializer(
+                    std=0.1
+                )
+            else:
+                # Use the identity matrix for each mode/state
+                initializer = osld_initializers.IdentityCholeskyInitializer()
+
+        # Create a layer to learn the covariance matrices
+        #
+        # We use self.layers for compatibility with
+        # initializers.reinitialize_model_weights
+        self.layers = [
+            LearnableTensorLayer(
+                shape=(n, m * (m + 1) // 2),
+                learn=learn,
+                initializer=initializer,
+                initial_value=initial_flattened_cholesky_factors,
+                regularizer=regularizer,
+                name=self.name + "_kernel",
             )
+        ]
 
-        # Regulariser
-        self.regularizer = regularizer
-
-    def add_regularization(self, covariances, inputs):
-        # Calculate regularisation
-        reg = self.regularizer(covariances)
-
-        # Calculate the scaling factor for the regularisation
-        batch_size = tf.cast(tf.shape(inputs)[0], tf.float32)
-        n_batches = self.regularizer.n_batches
-        scaling_factor = batch_size * n_batches
-        reg /= scaling_factor
-
-        # Add to the loss function
-        self.add_loss(reg)
-        self.add_metric(reg, name=self.name)
-
-    def build(self, input_shape):
-        self.flattened_cholesky_factors = self.add_weight(
-            "flattened_cholesky_factors",
-            shape=(self.n, self.m * (self.m + 1) // 2),
-            dtype=tf.float32,
-            initializer=self.flattened_cholesky_factors_initializer,
-            trainable=self.learn,
-        )
-        self.built = True
-
-    def call(self, inputs, training=None, **kwargs):
-        covariances = self.bijector(self.flattened_cholesky_factors)
+    def call(self, inputs, **kwargs):
+        learnable_tensor_layer = self.layers[0]
+        flattened_cholesky_factors = learnable_tensor_layer(inputs, **kwargs)
+        covariances = self.bijector(flattened_cholesky_factors)
         covariances = add_epsilon(covariances, self.epsilon, diag=True)
-        if self.regularizer is not None and training:
-            self.add_regularization(covariances, inputs)
         return covariances
 
 
@@ -569,9 +588,6 @@ class CorrelationMatricesLayer(layers.Layer):
         **kwargs,
     ):
         super().__init__(**kwargs)
-        self.n = n
-        self.m = m
-        self.learn = learn
         self.epsilon = epsilon
 
         # Bijector used to transform learnable vectors to correlation matrices
@@ -579,74 +595,50 @@ class CorrelationMatricesLayer(layers.Layer):
             [tfb.CholeskyOuterProduct(), tfb.CorrelationCholesky()]
         )
 
+        # Do we have an initial value?
         if initial_value is not None:
-            # Use passed value for initialisation
-            if initial_value.ndim != 3:
-                raise ValueError(
-                    "a (n_modes, n_channels, n_channels) array must be passed for "
-                    + "initial_fcs."
-                )
-            if initial_value.shape[-1] != m:
-                raise ValueError(
-                    f"mismatch between the number of channels ({m}) and number of "
-                    + "rows/columns in initial_fcs "
-                    + f"({initial_value.shape[-1]})."
-                )
-            if initial_value.shape[0] != n:
-                raise ValueError(
-                    "mismatch bettwen the number of modes and matrices in "
-                    + f"initial_fcs ({initial_value.shape[0]})."
-                )
+            # Check it's the correct shape
+            if initial_value.shape != (n, m, m):
+                raise ValueError(f"initial_value shape must be ({n}, {m}, {m}).")
+
+            # Calculate the flattened cholesky factors
             initial_value = initial_value.astype("float32")
             initial_flattened_cholesky_factors = self.bijector.inverse(initial_value)
-            self.flattened_cholesky_factors_initializer = (
-                osld_initializers.WeightInitializer(initial_flattened_cholesky_factors)
-            )
-
-        elif learn:
-            # Use a correlation matrix with an error added
-            self.flattened_cholesky_factors_initializer = (
-                osld_initializers.NormalCorrelationCholeskyInitializer(std=0.1)
-            )
 
+            # We don't need an initializer
+            initializer = None
         else:
-            # Use an identity matrix
-            self.flattened_cholesky_factors_initializer = (
-                osld_initializers.IdentityCholeskyInitializer()
+            # No initial value has been passed
+            initial_flattened_cholesky_factors = None
+            if learn:
+                # Use a random initializer
+                initializer = osld_initializers.NormalCorrelationCholeskyInitializer(
+                    std=0.1
+                )
+            else:
+                # Use the identity matrix for each mode/state
+                initializer = osld_initializers.IdentityCholeskyInitializer()
+
+        # Create a layer to learn the correlation matrices
+        #
+        # We use self.layers for compatibility with
+        # initializers.reinitialize_model_weights
+        self.layers = [
+            LearnableTensorLayer(
+                shape=(n, m * (m - 1) // 2),
+                learn=learn,
+                initializer=initializer,
+                initial_value=initial_flattened_cholesky_factors,
+                regularizer=regularizer,
+                name=self.name + "_kernel",
             )
+        ]
 
-        # Regulariser
-        self.regularizer = regularizer
-
-    def add_regularization(self, correlations, inputs):
-        # Calculate regularisation
-        reg = self.regularizer(correlations)
-
-        # Calculate the scaling factor for the regularisation
-        batch_size = tf.cast(tf.shape(inputs)[0], tf.float32)
-        n_batches = self.regularizer.n_batches
-        scaling_factor = batch_size * n_batches
-        reg /= scaling_factor
-
-        # Add to the loss function
-        self.add_loss(reg)
-        self.add_metric(reg, name=self.name)
-
-    def build(self, input_shape):
-        self.flattened_cholesky_factors = self.add_weight(
-            "flattened_cholesky_factors",
-            shape=(self.n, self.m * (self.m - 1) // 2),
-            dtype=tf.float32,
-            initializer=self.flattened_cholesky_factors_initializer,
-            trainable=self.learn,
-        )
-        self.built = True
-
-    def call(self, inputs, training=None, **kwargs):
-        correlations = self.bijector(self.flattened_cholesky_factors)
+    def call(self, inputs, **kwargs):
+        learnable_tensor_layer = self.layers[0]
+        flattened_cholesky_factors = learnable_tensor_layer(inputs, **kwargs)
+        correlations = self.bijector(flattened_cholesky_factors)
         correlations = add_epsilon(correlations, self.epsilon, diag=True)
-        if self.regularizer is not None and training:
-            self.add_regularization(correlations, inputs)
         return correlations
 
 
@@ -682,84 +674,59 @@ class DiagonalMatricesLayer(layers.Layer):
         **kwargs,
     ):
         super().__init__(**kwargs)
-        self.n = n
-        self.m = m
-        self.learn = learn
         self.epsilon = epsilon
 
         # Softplus transformation to ensure diagonal is positive
         self.bijector = tfb.Softplus()
 
+        # Do we have an initial value?
         if initial_value is not None:
-            # Used passed value for initialisation
-            if initial_value.ndim != 2:
+            # Check it's the correct shape
+            if initial_value.shape == (n, m, m):
+                # Keep the diagonal only
+                initial_value = np.diagonal(initial_value, axis1=1, axis2=2)
+            elif initial_value.shape != (n, m):
                 raise ValueError(
-                    "a (n_modes, n_channels) array must be passed for initial_value."
-                )
-            if initial_value.shape[-1] != m:
-                raise ValueError(
-                    f"mismatch between the number of channels ({m}) and number of "
-                    + f"elements in initial_value ({initial_value.shape[-1]})."
-                )
-            if initial_value.shape[0] != n:
-                raise ValueError(
-                    "mismatch bettwen the number of modes and vectors in "
-                    + f"initial_value ({initial_value.shape[0]})."
+                    f"initial_value shape must be ({n}, {m}, {m}) or ({n}, {m})."
                 )
+
+            # Calculate the initial value of the learnable tensor
             initial_value = initial_value.astype("float32")
             initial_diagonals = self.bijector.inverse(initial_value)
-            self.diagonals_initializer = osld_initializers.WeightInitializer(
-                initial_diagonals
-            )
-
-        elif learn:
-            # Use random numbers
-            self.diagonals_initializer = osld_initializers.NormalDiagonalInitializer(
-                std=0.05
-            )
 
+            # We don't need an initializer
+            initializer = None
         else:
-            # Use ones
-            initial_value = np.ones(size=[n, m]).astype(np.float32)
-            initial_diagonals = self.bijector.inverse(initial_value)
-            self.diagonals_initializer = osld_initializers.WeightInitializer(
-                initial_diagonals
+            # No initial value has been passed
+            initial_diagonals = None
+            if learn:
+                # Use a random initializer
+                initializer = osld_initializers.NormalDiagonalInitializer(std=0.05)
+            else:
+                # Use the identity matrix for each mode/state
+                initializer = osld_initializers.IdentityCholeskyInitializer()
+
+        # Create a layer to learn the matrices
+        #
+        # We use self.layers for compatibility with
+        # initializers.reinitialize_model_weights
+        self.layers = [
+            LearnableTensorLayer(
+                shape=(n, m),
+                learn=learn,
+                initializer=initializer,
+                initial_value=initial_diagonals,
+                regularizer=regularizer,
+                name=self.name + "_kernel",
             )
+        ]
 
-        # Regulariser
-        self.regularizer = regularizer
-
-    def add_regularization(self, diagonals, inputs):
-        # Calculate regularisation
-        reg = self.regularizer(diagonals)
-
-        # Calculate the scaling factor for the regularisation
-        batch_size = tf.cast(tf.shape(inputs)[0], tf.float32)
-        n_batches = self.regularizer.n_batches
-        scaling_factor = batch_size * n_batches
-        reg /= scaling_factor
-
-        # Add to the loss function
-        self.add_loss(reg)
-        self.add_metric(reg, name=self.name)
-
-    def build(self, input_shape):
-        self.diagonals = self.add_weight(
-            "diagonals",
-            shape=(self.n, self.m),
-            dtype=tf.float32,
-            initializer=self.diagonals_initializer,
-            trainable=self.learn,
-        )
-        self.built = True
-
-    def call(self, inputs, training=None, **kwargs):
-        D = self.bijector(self.diagonals)
-        D = add_epsilon(D, self.epsilon)
-        if self.regularizer is not None and training:
-            self.add_regularization(D, inputs)
-        D = tf.linalg.diag(D)
-        return D
+    def call(self, inputs, **kwargs):
+        learnable_tensor_layer = self.layers[0]
+        diagonals = learnable_tensor_layer(inputs, **kwargs)
+        diagonals = self.bijector(diagonals)
+        diagonals = add_epsilon(diagonals, self.epsilon)
+        return tf.linalg.diag(diagonals)
 
 
 class MatrixLayer(layers.Layer):
@@ -805,8 +772,10 @@ class MatrixLayer(layers.Layer):
         else:
             raise ValueError("Please use constraint='diagonal' or 'covariance.'")
 
+        self.layers = [self.matrix_layer]
+
     def call(self, inputs, **kwargs):
-        return self.matrix_layer(inputs)[0]
+        return self.matrix_layer(inputs, **kwargs)[0]
 
 
 class MixVectorsLayer(layers.Layer):
@@ -1218,48 +1187,26 @@ class CategoricalLogLikelihoodLossLayer(layers.Layer):
         return tf.expand_dims(nll_loss, axis=-1)
 
 
-class SubjectDevEmbeddingLayer(layers.Layer):
+class ConcatEmbeddingsLayer(layers.Layer):
     """Layer for getting the concatenated embeddings.
 
     The concatenated embeddings are obtained by concatenating subject embeddings
-    and mode spatial map embeddings. The concatenated embeddings are used to
-    generate subject specific deviations from the group spatial map.
-
-    Parameters
-    ----------
-    n_modes : int
-        Number of modes.
-    n_channels: int
-        Number of channels.
-    n_subjects : int
-        Number of subjects.
+    and mode spatial map embeddings.
     """
 
-    def __init__(
-        self,
-        n_modes,
-        n_channels,
-        n_subjects,
-        **kwargs,
-    ):
-        super().__init__(**kwargs)
-        self.n_modes = n_modes
-        self.n_channels = n_channels
-        self.n_subjects = n_subjects
-
     def call(self, inputs):
         subject_embeddings, mode_embeddings = inputs
-        subject_embedding_dim = subject_embeddings.shape[-1]
-        mode_embedding_dim = mode_embeddings.shape[-1]
+        n_subjects, subject_embedding_dim = subject_embeddings.shape
+        n_modes, mode_embedding_dim = mode_embeddings.shape
 
         # Match dimensions for concatenation
         subject_embeddings = tf.broadcast_to(
             tf.expand_dims(subject_embeddings, axis=1),
-            [self.n_subjects, self.n_modes, subject_embedding_dim],
+            [n_subjects, n_modes, subject_embedding_dim],
         )
         mode_embeddings = tf.broadcast_to(
             tf.expand_dims(mode_embeddings, axis=0),
-            [self.n_subjects, self.n_modes, mode_embedding_dim],
+            [n_subjects, n_modes, mode_embedding_dim],
         )
 
         # Concatenate the embeddings
@@ -1287,27 +1234,27 @@ class SubjectMapLayer(layers.Layer):
         self.epsilon = epsilon
         if which_map == "covariances":
             self.bijector = tfb.Chain([tfb.CholeskyOuterProduct(), tfb.FillScaleTriL()])
-        elif which_map != "means":
+        elif which_map == "means":
+            self.bijector = tfb.Identity()
+        else:
             raise ValueError("which_map must be one of 'means' and 'covariances'.")
 
     def call(self, inputs):
         group_map, dev = inputs
-
-        if self.which_map == "covariances":
-            group_map = self.bijector.inverse(group_map)
+        group_map = self.bijector.inverse(group_map)
 
         # Match dimensions for addition
         group_map = tf.expand_dims(group_map, axis=0)
         subject_map = tf.add(group_map, dev)
+        subject_map = self.bijector(subject_map)
 
         if self.which_map == "covariances":
-            subject_map = self.bijector(subject_map)
             subject_map = add_epsilon(subject_map, self.epsilon, diag=True)
 
         return subject_map
 
 
-class MixSubjectEmbeddingParametersLayer(layers.Layer):
+class MixSubjectSpecificParametersLayer(layers.Layer):
     """Class for mixing means and covariances for the
     subject embedding model.
 
@@ -1340,14 +1287,17 @@ class MixSubjectEmbeddingParametersLayer(layers.Layer):
         return m, C
 
 
-class SubjectMapKLDivergenceLayer(layers.Layer):
-    """Layer to calculate KL divergence between posterior and prior of
-    subject specific deviation.
+class StaticKLDivergenceLayer(layers.Layer):
+    """Layer to calculate KL divergence between Gamma posterior
+    and exponential prior for static parameters.
 
     Parameters
     ----------
     epsilon : float
         Error added to the standard deviations for numerical stability.
+    n_batches : int
+        Number of batches in the data. This is for calculating
+        the scaling factor.
     """
 
     def __init__(self, epsilon, n_batches=1, **kwargs):
@@ -1356,15 +1306,18 @@ class SubjectMapKLDivergenceLayer(layers.Layer):
         self.n_batches = n_batches
 
     def call(self, inputs, **kwargs):
-        data, inference_mu, inference_sigma, model_sigma = inputs
+        data, inference_alpha, inference_beta, model_beta = inputs
 
         # Add a small error for numerical stability
-        inference_sigma = add_epsilon(inference_sigma, self.epsilon)
-        model_sigma = add_epsilon(model_sigma, self.epsilon)
+        inference_alpha = add_epsilon(inference_alpha, self.epsilon)
+        inference_beta = add_epsilon(inference_beta, self.epsilon)
+        model_beta = add_epsilon(model_beta, self.epsilon)
 
         # Calculate the KL divergence
-        prior = tfp.distributions.Normal(loc=0.0, scale=model_sigma)
-        posterior = tfp.distributions.Normal(loc=inference_mu, scale=inference_sigma)
+        prior = tfp.distributions.Exponential(rate=model_beta)
+        posterior = tfp.distributions.Gamma(
+            concentration=inference_alpha, rate=inference_beta
+        )
         kl_loss = tfp.distributions.kl_divergence(
             posterior, prior, allow_nan_stats=False
         )
