@@ -2,6 +2,7 @@
 
 """
 
+import logging
 from dataclasses import dataclass
 
 import numpy as np
@@ -9,20 +10,22 @@ import tensorflow as tf
 from tensorflow.keras import layers
 
 import osl_dynamics.data.tf as dtf
-from osl_dynamics.models.mod_base import BaseModelConfig, ModelBase
-from osl_dynamics.models import dynemo_obs
 from osl_dynamics.inference import regularizers
 from osl_dynamics.inference.initializers import WeightInitializer
 from osl_dynamics.inference.layers import (
-    add_epsilon,
-    LogLikelihoodLossLayer,
-    VectorsLayer,
-    DiagonalMatricesLayer,
     CorrelationMatricesLayer,
-    MixVectorsLayer,
-    MixMatricesLayer,
+    DiagonalMatricesLayer,
+    LogLikelihoodLossLayer,
     MatMulLayer,
+    MixMatricesLayer,
+    MixVectorsLayer,
+    VectorsLayer,
+    add_epsilon,
 )
+from osl_dynamics.models import dynemo_obs
+from osl_dynamics.models.mod_base import BaseModelConfig, ModelBase
+
+_logger = logging.getLogger("osl-dynamics")
 
 
 @dataclass
@@ -128,7 +131,7 @@ class Config(BaseModelConfig):
         super().validate_dimension_parameters()
         if self.n_fc_modes is None:
             self.n_fc_modes = self.n_modes
-            print("Warning: n_fc_modes is None, set to n_modes.")
+            _logger.warning("n_fc_modes is None, set to n_modes.")
 
 
 class Model(ModelBase):
@@ -194,14 +197,13 @@ class Model(ModelBase):
             dynemo_obs.set_means_regularizer(self.model, training_dataset)
 
         if self.config.learn_stds:
-            set_stds_regularizer(self.model, training_dataset)
+            set_stds_regularizer(self.model, training_dataset, self.config.stds_epsilon)
 
         if self.config.learn_fcs:
-            set_fcs_regularizer(self.model, training_dataset)
+            set_fcs_regularizer(self.model, training_dataset, self.config.fcs_epsilon)
 
 
 def _model_structure(config):
-
     # Layers for inputs
     data = layers.Input(shape=(config.sequence_length, config.n_channels), name="data")
     alpha = layers.Input(shape=(config.sequence_length, config.n_modes), name="alpha")
@@ -272,14 +274,14 @@ def get_means_stds_fcs(model):
     stds_layer = model.get_layer("stds")
     fcs_layer = model.get_layer("fcs")
 
-    means = means_layer.vectors
+    means = means_layer(1)
     stds = add_epsilon(
-        tf.linalg.diag(stds_layer.bijector(stds_layer.diagonals)),
+        stds_layer(1),
         stds_layer.epsilon,
         diag=True,
     )
     fcs = add_epsilon(
-        fcs_layer.bijector(fcs_layer.flattened_cholesky_factors),
+        fcs_layer(1),
         fcs_layer.epsilon,
         diag=True,
     )
@@ -305,20 +307,20 @@ def set_means_stds_fcs(model, means, stds, fcs, update_initializer=True):
     flattened_cholesky_factors = fcs_layer.bijector.inverse(fcs)
 
     # Set values
-    means_layer.vectors.assign(means)
-    stds_layer.diagonals.assign(diagonals)
-    fcs_layer.flattened_cholesky_factors.assign(flattened_cholesky_factors)
+    means_layer.vectors_layer.tensor.assign(means)
+    stds_layer.diagonals_layer.tensor.assign(diagonals)
+    fcs_layer.flattened_cholesky_factors_layer.tensor.assign(flattened_cholesky_factors)
 
     # Update initialisers
     if update_initializer:
-        means_layer.vectors_initializer = WeightInitializer(means)
-        stds_layer.diagonals_initializer = WeightInitializer(diagonals)
-        fcs_layer.flattened_cholesky_factors_initializer = WeightInitializer(
-            flattened_cholesky_factors
+        means_layer.vectors_layer.tensor_initializer = WeightInitializer(means)
+        stds_layer.diagonals_layer.tensor_initializer = WeightInitializer(diagonals)
+        fcs_layer.flattened_cholesky_factors_layer.tensor_initializer = (
+            WeightInitializer(flattened_cholesky_factors)
         )
 
 
-def set_stds_regularizer(model, training_dataset):
+def set_stds_regularizer(model, training_dataset, epsilon):
     n_batches = dtf.get_n_batches(training_dataset)
     n_channels = dtf.get_n_channels(training_dataset)
     range_ = dtf.get_range(training_dataset)
@@ -327,18 +329,23 @@ def set_stds_regularizer(model, training_dataset):
     sigma = np.sqrt(np.log(2 * range_))
 
     stds_layer = model.get_layer("stds")
-    stds_layer.regularizer = regularizers.LogNormal(mu, sigma, n_batches)
+    learnable_tensor_layer = stds_layer.layers[0]
+    learnable_tensor_layer.regularizer = regularizers.LogNormal(
+        mu, sigma, epsilon, n_batches
+    )
 
 
-def set_fcs_regularizer(model, training_dataset):
+def set_fcs_regularizer(model, training_dataset, epsilon):
     n_batches = dtf.get_n_batches(training_dataset)
     n_channels = dtf.get_n_channels(training_dataset)
 
     nu = n_channels - 1 + 0.1
 
     fcs_layer = model.get_layer("fcs")
-    fcs_layer.regularizer = regularizers.MarginalInverseWishart(
+    learnable_tensor_layer = fcs_layer.layers[0]
+    learnable_tensor_layer.regularizer = regularizers.MarginalInverseWishart(
         nu,
+        epsilon,
         n_channels,
         n_batches,
     )
