@@ -39,7 +39,6 @@ class HMM:
         n_states=None,
         random_seed=None,
     ):
-
         if isinstance(trans_prob, list):
             trans_prob = np.ndarray(trans_prob)
 
@@ -466,6 +465,8 @@ class MSubj_HMM_MVN(Simulation):
         Number of subjects.
     n_groups : int
         Number of groups of subjects when subject means or covariances are 'random'.
+    between_group_scale : float
+        Scale of variability between subject observation parameters.
     stay_prob : float
         Used to generate the transition probability matrix is trans_prob is a str.
     subject_tc_std : float
