@@ -5,9 +5,11 @@
 import logging
 from itertools import zip_longest
 
+import numpy as np
+import pandas as pd
+import seaborn as sns
 import matplotlib
 import matplotlib.pyplot as plt
-import numpy as np
 from matplotlib import patches
 from matplotlib.path import Path
 from mpl_toolkits.axes_grid1.axes_divider import make_axes_locatable
@@ -21,21 +23,6 @@ from osl_dynamics.utils.parcellation import Parcellation
 
 _logger = logging.getLogger("osl-dynamics")
 
-QUAL_CMAPS = [
-    "Pastel1",
-    "Pastel2",
-    "Paired",
-    "Accent",
-    "Dark2",
-    "Set1",
-    "Set2",
-    "Set3",
-    "tab10",
-    "tab20",
-    "tab20b",
-    "tab20c",
-]
-
 
 def set_style(params):
     """Sets matplotlib's style.
@@ -81,7 +68,7 @@ def show(tight_layout=True):
 
 
 def save(fig, filename, tight_layout=True):
-    """Saves a figure.
+    """Save and close a figure.
 
     Parameters
     ----------
@@ -96,6 +83,7 @@ def save(fig, filename, tight_layout=True):
     if tight_layout:
         fig.tight_layout()
     fig.savefig(filename)
+    close(fig)
 
 
 def close(fig=None):
@@ -216,9 +204,9 @@ def plot_line(
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object. Only returned if ax=None.
+        Matplotlib figure object. Only returned if ax=None and filename=None.
     ax : matplotlib.pyplot.axis.
-        Matplotlib axis object(s). Only returned if ax=None.
+        Matplotlib axis object(s). Only returned if ax=None and filename=None.
     """
 
     # Validation
@@ -302,8 +290,7 @@ def plot_line(
     # Save figure
     if filename is not None:
         save(fig, filename)
-
-    if create_fig:
+    elif create_fig:
         return fig, ax
 
 
@@ -365,9 +352,9 @@ def plot_scatter(
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object. Only returned if ax=None.
+        Matplotlib figure object. Only returned if ax=None and filename=None.
     ax : matplotlib.pyplot.axis.
-        Matplotlib axis object(s). Only returned if ax=None.
+        Matplotlib axis object(s). Only returned if ax=None and filename=None.
     """
 
     # Validation
@@ -466,8 +453,7 @@ def plot_scatter(
     # Save figure
     if filename is not None:
         save(fig, filename)
-
-    if create_fig:
+    elif create_fig:
         return fig, ax
 
 
@@ -520,9 +506,9 @@ def plot_hist(
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object. Only returned if ax=None.
+        Matplotlib figure object. Only returned if ax=None and filename=None.
     ax : matplotlib.pyplot.axis.
-        Matplotlib axis object(s). Only returned if ax=None.
+        Matplotlib axis object(s). Only returned if ax=None and filename=None.
     """
 
     # Validation
@@ -589,8 +575,7 @@ def plot_hist(
     # Save the figure if a filename has been pass
     if filename is not None:
         save(fig, filename)
-
-    if create_fig:
+    elif create_fig:
         return fig, ax
 
 
@@ -637,9 +622,9 @@ def plot_bar_chart(
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object. Only returned if ax=None.
+        Matplotlib figure object. Only returned if ax=None and filename=None.
     ax : matplotlib.pyplot.axis.
-        Matplotlib axis object(s). Only returned if ax=None.
+        Matplotlib axis object(s). Only returned if ax=None and filename=None.
     """
 
     # Validation
@@ -694,8 +679,7 @@ def plot_bar_chart(
     # Save the figure if a filename has been pass
     if filename is not None:
         save(fig, filename)
-
-    if create_fig:
+    elif create_fig:
         return fig, ax
 
 
@@ -722,6 +706,9 @@ def plot_gmm(
     ----------
     data : np.ndarray
         Raw data to plot as a histogram.
+    amplitudes : np.ndarray
+        Amplitudes of each Gaussian component.
+        Mixture weights scaled by mixture covariances.
     means : np.ndarray
         Mean of each Gaussian component.
     stddevs : np.ndarray
@@ -752,9 +739,9 @@ def plot_gmm(
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object. Only returned if ax=None.
+        Matplotlib figure object. Only returned if ax=None and filename=None.
     ax : matplotlib.pyplot.axis.
-        Matplotlib axis object(s). Only returned if ax=None.
+        Matplotlib axis object(s). Only returned if ax=None and filename=None.
     """
 
     # Validation
@@ -788,19 +775,21 @@ def plot_gmm(
         fig, ax = create_figure(**fig_kwargs)
 
     # Plot histogram
-    ax.hist(data, bins=bins, histtype="step", density=True)
+    ax.hist(data, bins=bins, histtype="step", density=True, color="tab:blue")
 
     # Plot Gaussian components
     x = np.arange(min(data), max(data), (max(data) - min(data)) / bins)
     y1 = amplitudes[0] * np.exp(-((x - means[0]) ** 2) / (2 * stddevs[0] ** 2))
     y2 = amplitudes[1] * np.exp(-((x - means[1]) ** 2) / (2 * stddevs[1] ** 2))
-    ax.plot(x, y1, label="Off")
-    ax.plot(x, y2, label="On")
-    ax.plot(x, y1 + y2)
+    ax.plot(x, y1, label="Off", color="tab:orange")
+    ax.plot(x, y2, label="On", color="tab:green")
+    ax.plot(x, y1 + y2, color="tab:red")
 
     # Set axis range
-    ax.set_xlim(x_range[0], x_range[1])
-    ax.set_ylim(y_range[0], y_range[1])
+    if not any(r is None for r in x_range):
+        ax.set_xlim(x_range[0], x_range[1])
+    if not any(r is None for r in y_range):
+        ax.set_ylim(y_range[0], y_range[1])
 
     # Set title and axis labels
     ax.set_title(title)
@@ -808,28 +797,22 @@ def plot_gmm(
     ax.set_ylabel(y_label)
 
     # Add legend
-    ax.legend(loc=legend_loc)
+    if legend_loc is not None:
+        ax.legend(loc=legend_loc)
 
     # Save the figure if a filename has been pass
     if filename is not None:
         save(fig, filename)
-
-    if create_fig:
+    elif create_fig:
         return fig, ax
 
 
 def plot_violin(
     data,
-    show_mean=True,
-    show_median=True,
-    legend_loc=1,
     x=None,
-    x_range=None,
-    y_range=None,
     x_label=None,
     y_label=None,
     title=None,
-    plot_kwargs=None,
     fig_kwargs=None,
     ax=None,
     filename=None,
@@ -840,26 +823,14 @@ def plot_violin(
     ----------
     data : list of np.ndarray
         Data to plot.
-    show_mean : bool
-        Should we show the mean?
-    show_median : bool
-        Should we show the median?
-    legend_loc : int
-        Position for the legend.
     x : list or np.ndarray
         x-values for data.
-    x_range : list
-        Minimum and maximum for x-axis.
-    y_range : list
-        Minimum and maximum for y-axis.
     x_label : str
         Label for x-axis.
     y_label : str
         Label for y-axis.
     title : str
         Figure title.
-    plot_kwargs : dict
-        Arguments to pass to the ax.violinplot method.
     fig_kwargs : dict
         Arguments to pass to plt.subplots.
     ax : matplotlib.axes.axes
@@ -870,25 +841,19 @@ def plot_violin(
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object. Only returned if ax=None.
+        Matplotlib figure object. Only returned if ax=None and filename=None.
     ax : matplotlib.pyplot.axis.
-        Matplotlib axis object(s). Only returned if ax=None.
+        Matplotlib axis object(s). Only returned if ax=None and filename=None.
     """
 
     # Validation
     if x is None:
-        x = range(len(data))
+        x = np.arange(len(data)) + 1
     elif len(x) != len(data):
         raise ValueError("Incorrect number of x-values or data passed.")
     else:
         x = [str(xi) for xi in x]
 
-    if x_range is None:
-        x_range = [None, None]
-
-    if y_range is None:
-        y_range = [None, None]
-
     if ax is not None:
         if filename is not None:
             raise ValueError(
@@ -904,11 +869,13 @@ def plot_violin(
     else:
         fig_kwargs = override_dict_defaults(default_fig_kwargs, fig_kwargs)
 
-    if plot_kwargs is None:
-        plot_kwargs = {}
-
-    # Replace emtpy lists in data with a pair of nans
-    data = [np.array([np.nan, np.nan]) if len(d) == 0 else d for d in data]
+    # Create a pandas DataFrame
+    data_dict = {}
+    for x, d in zip(x, data):
+        data_dict[x] = []
+        for y in d:
+            data_dict[x].append(y)
+    df = pd.DataFrame(data_dict)
 
     # Create figure
     create_fig = ax is None
@@ -916,34 +883,17 @@ def plot_violin(
         fig, ax = create_figure(**fig_kwargs)
 
     # Plot violins
-    ax.violinplot(data, positions=range(len(x)), showextrema=False)
-    if show_mean:
-        ax.scatter(
-            x, [np.mean(d) for d in data], label="Mean", marker="+", color="black"
-        )
-    if show_median:
-        ax.scatter(
-            x, [np.median(d) for d in data], label="Median", marker="x", color="black"
-        )
-
-    # Set axis range
-    ax.set_xlim(x_range[0], x_range[1])
-    ax.set_ylim(y_range[0], y_range[1])
+    sns.violinplot(df, ax=ax)
 
     # Set title and axis labels
     ax.set_title(title)
     ax.set_xlabel(x_label)
     ax.set_ylabel(y_label)
 
-    # Add a legend
-    if show_mean and show_median:
-        ax.legend(loc=legend_loc)
-
     # Save the figure if a filename has been pass
     if filename is not None:
         save(fig, filename)
-
-    if create_fig:
+    elif create_fig:
         return fig, ax
 
 
@@ -978,9 +928,9 @@ def plot_time_series(
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object. Only returned if ax=None.
+        Matplotlib figure object. Only returned if ax=None and filename=None.
     ax : matplotlib.pyplot.axis.
-        Matplotlib axis object(s). Only returned if ax=None.
+        Matplotlib axis object(s). Only returned if ax=None and filename=None.
     """
     time_series = np.asarray(time_series)
     n_samples = min(n_samples or np.inf, time_series.shape[0])
@@ -1037,8 +987,7 @@ def plot_time_series(
     # Save figure
     if filename is not None:
         save(fig, filename)
-
-    if create_fig:
+    elif create_fig:
         return fig, ax
 
 
@@ -1071,9 +1020,9 @@ def plot_separate_time_series(
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object.
+        Matplotlib figure object. Only returned if filename=None.
     ax : matplotlib.pyplot.axis.
-        Matplotlib axis object(s).
+        Matplotlib axis object(s). Only returned if filename=None.
     """
     time_series = np.asarray(time_series)
     n_samples = n_samples or min([ts.shape[0] for ts in time_series])
@@ -1116,8 +1065,8 @@ def plot_separate_time_series(
     # Save figure
     if filename is not None:
         save(fig, filename)
-
-    return fig, axes
+    else:
+        return fig, axes
 
 
 def plot_epoched_time_series(
@@ -1169,9 +1118,9 @@ def plot_epoched_time_series(
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object. Only returned if ax=None.
+        Matplotlib figure object. Only returned if ax=None and filename=None.
     ax : matplotlib.pyplot.axis.
-        Matplotlib axis object(s). Only returned if ax=None.
+        Matplotlib axis object(s). Only returned if ax=None and filename=None.
     """
     from osl_dynamics.data.task import epoch_mean
 
@@ -1229,8 +1178,7 @@ def plot_epoched_time_series(
     # Save the figure if a filename has been passed
     if filename is not None:
         save(fig, filename)
-
-    if create_fig:
+    elif create_fig:
         return fig, ax
 
 
@@ -1272,9 +1220,9 @@ def plot_matrices(
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object.
+        Matplotlib figure object. Only returned if filename=None.
     ax : matplotlib.pyplot.axis.
-        Matplotlib axis object(s).
+        Matplotlib axis object(s). Only returned if filename=None.
     """
     matrix = np.array(matrix)
     if matrix.ndim == 2:
@@ -1332,8 +1280,8 @@ def plot_matrices(
 
     if filename is not None:
         save(fig, filename, tight_layout=False)
-
-    return fig, axes
+    else:
+        return fig, axes
 
 
 def plot_connections(
@@ -1371,9 +1319,9 @@ def plot_connections(
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object.
+        Matplotlib figure object. Only returned if filename=None.
     ax : matplotlib.pyplot.axis.
-        Matplotlib axis object(s).
+        Matplotlib axis object(s). Only returned if filename=None.
     """
     weights = np.abs(weights)
     x, y = np.diag_indices_from(weights)
@@ -1503,8 +1451,8 @@ def plot_connections(
 
     if filename is not None:
         save(fig, filename)
-
-    return fig, ax
+    else:
+        return fig, ax
 
 
 def topoplot(
@@ -1559,7 +1507,7 @@ def topoplot(
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object.
+        Matplotlib figure object. Only returned if filename=None.
     """
     topology = Topology(layout)
 
@@ -1580,8 +1528,8 @@ def topoplot(
 
     if filename is not None:
         save(fig, filename)
-
-    return fig
+    else:
+        return fig
 
 
 def plot_brain_surface(
@@ -1657,6 +1605,7 @@ def plot_alpha(
     plot_kwargs=None,
     fig_kwargs=None,
     filename=None,
+    axes=None,
 ):
     """Plot alpha.
 
@@ -1680,18 +1629,38 @@ def plot_alpha(
         Arguments to pass to matplotlib.pyplot.subplots.
     filename : str
         Output filename.
+    axes: list of matplotlib.pyplot.Axes
+        A list of matplotlib axes to plot on. If None, a new figure is created.
 
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object.
+        Matplotlib figure object. Only returned if filename=None.
     ax : matplotlib.pyplot.axis.
-        Matplotlib axis object(s).
+        Matplotlib axis object(s). Only returned if filename=None.
     """
     n_alphas = len(alpha)
+    if isinstance(axes, plt.Axes):
+        axes = [axes]
+    if axes is not None and len(axes) != n_alphas:
+        raise ValueError("Number of axes must match number of alphas.")
+
     n_modes = max(a.shape[1] for a in alpha)
     n_samples = min(n_samples or np.inf, alpha[0].shape[0])
-    if cmap in QUAL_CMAPS:
+    if cmap in [
+        "Pastel1",
+        "Pastel2",
+        "Paired",
+        "Accent",
+        "Dark2",
+        "Set1",
+        "Set2",
+        "Set3",
+        "tab10",
+        "tab20",
+        "tab20b",
+        "tab20c",
+    ]:
         cmap = plt.cm.get_cmap(name=cmap)
     else:
         cmap = plt.cm.get_cmap(name=cmap, lut=n_modes)
@@ -1719,11 +1688,13 @@ def plot_alpha(
     elif len(y_labels) != n_alphas:
         raise ValueError("Incorrect number of y_labels passed.")
 
-    # Create figure
-    fig, axes = create_figure(n_alphas, **fig_kwargs)
+    # Create figure if axes not passed
+    if axes is None:
+        fig, axes = create_figure(n_alphas, **fig_kwargs)
+    else:
+        fig = axes[0].get_figure()
 
-    # If n_alphas is one then axes won't be iterable
-    if not isinstance(axes, np.ndarray):
+    if isinstance(axes, plt.Axes):
         axes = [axes]
 
     # Plot data
@@ -1757,8 +1728,8 @@ def plot_alpha(
     # Save to file if a filename as been passed
     if filename is not None:
         save(fig, filename, tight_layout=False)
-
-    return fig, axes
+    else:
+        return fig, axes
 
 
 def plot_mode_lifetimes(
@@ -1808,9 +1779,9 @@ def plot_mode_lifetimes(
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object.
+        Matplotlib figure object. Only returned if filename=None.
     ax : matplotlib.pyplot.axis.
-        Matplotlib axis object(s).
+        Matplotlib axis object(s). Only returned if filename=None.
     """
     from osl_dynamics.analysis import modes
 
@@ -1892,8 +1863,8 @@ def plot_mode_lifetimes(
     # Save file is a filename has been passed
     if filename is not None:
         save(fig, filename)
-
-    return fig, axes
+    else:
+        return fig, axes
 
 
 def plot_psd_topo(
@@ -1920,9 +1891,9 @@ def plot_psd_topo(
     Returns
     -------
     fig : matplotlib.pyplot.figure
-        Matplotlib figure object.
+        Matplotlib figure object. Only returned if filename=None.
     ax : matplotlib.pyplot.axis.
-        Matplotlib axis object(s).
+        Matplotlib axis object(s). Only returned if filename=None.
     """
 
     if parcellation_file is not None:
@@ -1933,14 +1904,14 @@ def plot_psd_topo(
         # Re-order to use colour to indicate anterior->posterior location
         order = np.argsort(roi_centers[:, 1])
         roi_centers = roi_centers[order]
-        psd = np.copy(psd)[order[::-1]]
+        psd = np.copy(psd)[order]
 
     n_parcels = psd.shape[0]
 
     # Plot PSDs
     fig, ax = create_figure()
-    cmap = plt.get_cmap()
-    for i in range(n_parcels):
+    cmap = plt.cm.viridis_r
+    for i in reversed(range(n_parcels)):
         ax.plot(f, psd[i], c=cmap(i / n_parcels))
     ax.set_xlabel("Frequency (Hz)")
     ax.set_ylabel("PSD (a.u.)")
@@ -1961,5 +1932,49 @@ def plot_psd_topo(
     # Save
     if filename is not None:
         save(fig, filename, tight_layout=False)
+    else:
+        return fig, ax
 
-    return fig, ax
+
+def plot_summary_stats_group_diff(name, summary_stats, pvalues, assignments, filename):
+    """Plot summary statistics for two groups as violin plots.
+
+    Parameters
+    ----------
+    name : str
+        Name of the summary statistic.
+    summary_stats : np.ndarray
+        Summary statistics. Shape is (n_subjects, n_states).
+    pvalues : np.ndarray
+        p-values for each summary statistic difference. Shape is (n_states,).
+    assignments : np.ndarray
+        Array of 1s and 2s indicating group assignment. Shape is (n_subjects,).
+    filename : str
+        Output filename.
+    """
+    # Create a pandas DataFrame to hold the summary stats
+    ss_dict = {name: [], "State": [], "Group": []}
+    n_subjects, n_states = summary_stats.shape
+    for subject in range(n_subjects):
+        for state in range(n_states):
+            ss_dict[name].append(summary_stats[subject, state])
+            ss_dict["State"].append(state + 1)
+            ss_dict["Group"].append(int(assignments[subject]))
+    ss_df = pd.DataFrame(ss_dict)
+
+    # Plot a half violin for each group
+    sns.violinplot(data=ss_df, x="State", y=name, hue="Group", split=True)
+
+    # Add a star above the violin to indicate significance
+    scatter_kwargs = {"c": "black", "s": 32, "marker": "*"}
+    for i in range(n_states):
+        if pvalues[i] < 0.01:
+            plt.scatter(i - 0.075, summary_stats[:, i].max() * 1.6, **scatter_kwargs)
+            plt.scatter(i + 0.075, summary_stats[:, i].max() * 1.6, **scatter_kwargs)
+        elif pvalues[i] < 0.05:
+            plt.scatter(i, summary_stats[:, i].max() * 1.6, **scatter_kwargs)
+
+    # Save
+    _logger.info(f"Saving {filename}")
+    plt.savefig(filename)
+    plt.close()
